#
# This module contains all of the code to initialize the user interface and
# start the system running.
#

package require log
package require sqlite3
package require mime
package require smtp
package require md4
package require cmdline

namespace eval ::mailroom {
  # Static variables
  variable appVersion 0.5.1
  variable appTitle Mailroom
  variable appUrl {http://jeremy.cowgar.com/mailroom/}
  
  # Command line options
  variable params
  variable mailstore 0
  variable debug 0

  # Internal configs
  variable mailroom_dir 0
  variable has_user_callbacks 0

  # Initialize Mailroom by:
  #   1. Parse/handle command line options
  #   2. Find the mailstore file
  #   3. Load an user code
  #   4. Open/Create the database
  #
  proc initialize {{additionalOptions {}}} {
    variable mailstore
    variable mailroom_dir
    variable debug
    variable has_user_callbacks
    variable db_version
    variable params
    
    global argv
    
    # Setup our logging
    if {[package provide Tk]==""} {
      ::log::lvCmdForall ::mailroom::log_append
    } else {
      ::log::lvCmdForall ::mailroom::gui_log_append
    }
    foreach level {debug info} {
      ::log::lvSuppress $level 1
    }
    foreach level {alert critical emergency error notice warning} {
      ::log::lvSuppress $level 0
    }
    
    # Parse our options
    set options [concat $additionalOptions \
      {
        {debug           "enable debugging"}
        {mailstore.arg 0 "specify a non-standard mailstore"}
      }]
    
    set usage ": mailroom \[options]\noptions:"
    if {[catch {array set params [::cmdline::getoptions argv $options $usage]} msg]} {
      puts $msg
      exit 1
    }
    set mailstore $params(mailstore)
    set debug $params(debug)
    
    # Make it work as a portable application, check the current directory for a .mailroom.::mailroom::db file.
    # If it exists *and* the user hasn't specified a different directory, use it.
    #
    if {$mailstore==0} {
      if {[file exists ".mailroom.db"]} {
        set mailstore ".mailroom.db"
      } else {
        set mailstore [glob ~]/.mailroom.db
      }
    }
    
    set mailroom_dir [file rootname $mailstore]
    if {[catch {file mkdir $mailroom_dir}]} {
      error_message "Could not make the temporary mailroom directory: $mailroom_dir"
      exit 1
    }

    # Include any user code
    #
    if {[catch {source [file rootname $mailstore]_user.tcl} result]} {
      if {[string match "*no such file*" $result]==0} {
        error_message $result "User callback error"
      }
    } else {
      set has_user_callbacks 1
    }
    
    sqlite3 ::mailroom::db $mailstore
    db timeout 4000
    
    if {[catch {::mailroom::config option internal dbver} dbVersionCurrent]} {
      set dbVersionCurrent {}
    }
    if {$dbVersionCurrent!=$db_version} {
      db transaction {
        if {$dbVersionCurrent==""} {
          # Database schema does not yet exist, create the whole thing
          db eval $::mailroom::db_schema
        } else {
          # Use an array for the migration, not individual vars
          for {set newVersion [expr $dbVersionCurrent + 1]} {$newVersion <= $db_version} {incr newVersion} {
            db eval [set db_upgrade_$newVersion]
          }
          
          ::mailroom::config option internal dbver -value $db_version
        }
      }
    }
    
    # Create our in-memory tables
    #
    db eval $mailroom::memdb_schema

    # Setup debugging. Do not allow the database configuration to override specifying a -debug on the
    # command line.
    #
    if {$debug == 0} {
      set debug [::mailroom::config option config debug]
    }
    if {$debug == 1} {
      ::log::lvSuppress debug 0
    }
  }
  
  proc shutdown {} {
    if {$::mailroom::mailroom_dir!=0} {
      catch {file delete -force $::mailroom::mailroom_dir}
    }
    
    ::mailroom::db close
  }
  
  proc log_append {level msg} {
    puts stderr "$level \[[clock format [clock seconds] -format {%a %H:%M:%S}]] $msg"
  }
  
  proc gui_log_append {level msg} {
    .nb.log.t insert end "$level \[[clock format [clock seconds] -format {%a %H:%M:%S}]] $msg\n"
    .nb.log.t see end
    update
  }
  
  proc error_message {msg {title "Error"}} {
    log::log error $msg
    
    if {[package provide Tk]!=""} {
      tk_messageBox -icon error -title $title -message $title -detail $msg
    }
  }
}
